/* Copyright (c) 1997, MpegTV LLC */
#ifndef _audio_api_h
#define _audio_api_h
#ifndef lint
static const char rcsid_audio_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/audio_api.h,v 1.16 1999/05/19 10:16:31 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"

/*--------------------------------------------------------------------------+
| Data types:																|
+--------------------------------------------------------------------------*/

/* audio layer (DO NOT CHANGE! This must match MPEG-defined values)*/
typedef enum {
	MTVP_AUDIO_LAYER_1 = 3,
	MTVP_AUDIO_LAYER_2 = 2,
	MTVP_AUDIO_LAYER_3 = 1,
	MTVP_AUDIO_LAYER_RESERVED = 0
} MTVP_AUDIO_LAYER;

/* MPEG-1 audio mode (DO NOT CHANGE! This must match MPEG-defined values) */
typedef enum {
	MTVP_AUDIO_MODE_STEREO = 0,			/* stereo */
	MTVP_AUDIO_MODE_JOINT_STEREO = 1,		/* joint stereo */
	MTVP_AUDIO_MODE_DUAL_CHANNEL = 2,		/* dual channel  */
	MTVP_AUDIO_MODE_SINGLE_CHANNEL = 3		/* single channel */
} MTVP_AUDIO_MODE;

/* audio mode for the player (specified at startup via option -a<n>) */
typedef enum {
	MTVP_AUDIO_MODE_NONE,	/* no audio (no audio process) */
	MTVP_AUDIO_MODE_PLAY,	/* play audio (default) */
#ifdef AUDIO_MODE_STDOUT_SUPPORT
	MTVP_AUDIO_MODE_STDOUT,	/* print output audio data on stdout (no audio process) */
#endif
	MTVP_AUDIO_MODE_COUNT	/* number of audio modes modes */
} AUDIO_MODE;

/* audio channel selection */
typedef enum {
	MTVP_AUDIO_CHANNEL_BOTH,			/* both channels */
	MTVP_AUDIO_CHANNEL_LEFT,			/* left channel */
	MTVP_AUDIO_CHANNEL_RIGHT,			/* right channel */
	MTVP_AUDIO_CHANNEL_COUNT			/* number of audio channel selections */
} MTVP_AUDIO_CHANNEL;

/* audio quality */
typedef enum {
	MTVP_AUDIO_QUALITY_LOW,		/* low (ulaw 8 KHz) */
	MTVP_AUDIO_QUALITY_MEDIUM,	/* medium (half sample rate) */
	MTVP_AUDIO_QUALITY_HIGH,		/* high (full sample rate) */
	MTVP_AUDIO_QUALITY_COUNT		/* number of audio quality levels */
} MTVP_AUDIO_QUALITY;

/* audio infos */
typedef struct {
	MTVP_AUDIO_MODE	mode;
	MTVP_AUDIO_LAYER	layer;
	u_int		bitrate;		/* in bit/sec */
	double	sampling_frequency;	/* in Hz */
	double	output_frequency;	/* in Hz */
	Bool copyright;
	Bool original;
	Bool protected_flag;
} MTVP_PMP_DATA_AUDIO_INFOS;

/* audio controls selection */
typedef enum {
	MTVP_AUDIO_CONTROLS_MASTER,		/* master controls */
	MTVP_AUDIO_CONTROLS_PCM,			/* PCM controls */
	MTVP_AUDIO_CONTROLS_COUNT		/* number of audio controls selections */
} MTVP_AUDIO_CONTROLS;

/* audio output port selections */
typedef enum {
     MTVP_AUDIO_PORT_BIT_SPEAKER = 0,	/* output to built-in speaker */
     MTVP_AUDIO_PORT_BIT_HEADPHONE = 1,	/* output to headphone jack */
     MTVP_AUDIO_PORT_BIT_LINE_OUT = 2,	/* output to line out */
     MTVP_AUDIO_PORT_BIT_COUNT = 3		/* number of audio output port selections */
} MTVP_AUDIO_PORT_BIT;

/* audio output port selection masks */
typedef enum {
     MTVP_AUDIO_PORT_SPEAKER = (1 << MTVP_AUDIO_PORT_BIT_SPEAKER),	/* output to built-in speaker */
     MTVP_AUDIO_PORT_HEADPHONE = (1 << MTVP_AUDIO_PORT_BIT_HEADPHONE),/* output to headphone jack */
     MTVP_AUDIO_PORT_LINE_OUT = (1 << MTVP_AUDIO_PORT_BIT_LINE_OUT)	/* output to line out */
} MTVP_AUDIO_PORT;


#endif /* _audio_api_h */
