/* Copyright (c) 1998, MpegTV LLC */
#ifndef _portable_h
#define _portable_h
#ifndef lint
static const char rcsid_portable_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/lib/include/RCS/portable.h,v 1.11 1999/09/08 12:53:34 tristan Exp $";
#endif

/*--------------------------------------------------------------------------+
| The "portable" package provides routines to read and write 8-bit chars,	|
| and 16-bit and 32-bit integers from/to a FILE, to get and set the			|
| size of a FILE in a very portable way.									|
|																			|
| It performs the same way on any architecture (little endian, big endian), |
| whereas the regular routines (fread, fwrite) don't.						|
+--------------------------------------------------------------------------*/

/* we may need to include some non-POSIX.1 things here... */

/* undef the flag _POSIX_SOURCE */
#ifdef _POSIX_SOURCE
#	undef NO_POSIX_SOURCE
#	undef _POSIX_SOURCE
#endif

#if 0
///* define struct timeval here */
//#ifndef INCLUDE_FOR_STRUCT_TIMEVAL
//
//struct timeval {
//	long	tv_sec;		/* seconds */
//	long	tv_usec;	/* and microseconds */
//};
//
//#endif /* not INCLUDE_FOR_STRUCT_TIMEVAL */
#endif /* 0 */

#ifdef SGI
/* to define struct timeval */
#	include <utmpx.h>
#endif

#ifndef NO_AIO

#include <sys/asynch.h>	/* asynchronous IO routines, not POSIX.1 */

/* external declarations forgotten in standard include files */
#if defined(CONFIG_sun4) || defined(CONFIG_sun4gcc)
extern int aiowrite(int, const char *, int, int, int, aio_result_t *);
extern int aioread(int, char *, int, int, int, aio_result_t *);
extern aio_result_t * aiowait(struct timeval *);
#endif /* defined(CONFIG_sun4) || defined(CONFIG_sun4gcc) */
#endif /* not NO_AIO */

#include "misc.h"

#ifdef NO_HOSTNAME

#	include <sys/utsname.h>
#	define MAXHOSTNAMLENTH	SYS_NMLN

#else /* not NO_HOSTNAME */

#ifndef WIN32
	/*
	 * This trick is to includes <sys/param.h>,
	 * which is not part of the POSIX.1 Interface.
	 * <sys/param.h> defines MAXHOSTNAMLENTH and gethostname().
	 * This trick works on the Sun undes SunOS 4.x.
	 * I hope it works also on other machines...
	 */

	/* define __sys_signal_h to avoid including <sys/signal.h> */ 
#	define __sys_signal_h

#	include <sys/param.h>

#endif /* not WIN32 */

#endif /* not NO_HOSTNAME */


/* restore the flag _POSIX_SOURCE */
#ifndef NO_POSIX_SOURCE
#	define _POSIX_SOURCE
#endif



/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/*
 * Bit shift right with signed extension.
 * This is equivalent to divide by a power of 2 (with rounding toward minus infinity).
 * Note: x>>n is implementation dependent when x < 0 and n >= 0 !!
 * (x) must be a signed quantity, (n) must be >=0 or unsigned,
 * and (n) must be < 8 * sizeof(x).
 */
#ifdef SHIFT_RIGHT_EXTENDS_SIGN
#	define	PORTABLE_SIGNED_SHIFT_RIGHT(x, n)	((x)>>(n))
#else
#if 0
#	define	PORTABLE_SIGNED_SHIFT_RIGHT(x, n)	((x)>=0 ? ((x)>>(n)) : -((-(x)+1)>>(n)))
#endif /* 0 */
	/* the following macro should be correct.  -((-(x)+1)>>(n)) was wrong... */
#	define	PORTABLE_SIGNED_SHIFT_RIGHT(x, n)	((x)>=0 ? ((x)>>(n)) : ~((~(x))>>(n)))
#endif

/*
 * Divide by a power of 2 (with rounding toward 0).
 * Note: x>>n and x/(1<<n) are implementation dependent when x < 0 !!
 * (x) must be a signed quantity, (n) must be >=0 or unsigned,
 * and (n) must be < 8 * sizeof(x).
 */
#define	PORTABLE_SIGNED_DIVIDE_BY_POWER_OF_2(x, n) ((x)>=0 ? ((x)>>(n)) : -((-(x))>>(n)))

/*
 * PORTABLE_GET_?() reads a signed value. ? can be equal to 1, 2 or 4 (bytes).
 * PORTABLE_UGET_?() reads a unsigned value. ? can be equal to 1, 2 or 4 (bytes).
 *
 * To test if an error occured, ferror() must be used.
 *
 * PORTABLE_PUT() and PORTABLE_GET() are now defined as macros.
 *
 * The last argument of those macros *MUST* be 1, 2, or 4, and should
 * not be preceded (or followed) by any blanck space or comments.
 *
 * Example: PORTABLE_GET(fd, 2,&x) ===> incorrect
 *          PORTABLE_GET(fd,2,&x) ===> ok
 */

#define PORTABLE_GET_1(fd, x)		STMT( \
		*(x) = (char)fgetc(fd); \
	)
	
#define PORTABLE_UGET_1(fd, x)		STMT( \
		*(x) = (u_char)fgetc(fd); \
	)
	
#if 0
//#define PORTABLE_GET_2(fd)		_portable_get_2(fd)
//#define PORTABLE_UGET_2(fd)		_portable_uget_2(fd)
//#define PORTABLE_GET_4(fd)		_portable_get_4(fd)
//#define PORTABLE_UGET_4(fd)		_portable_uget_4(fd)
#endif /* 0 */


#define PORTABLE_PUT_1(fd, x)	\
	STMT( \
		fputc((int)(x), fd); \
	)

#define PORTABLE_UPUT_1(fd, x)	\
	STMT( \
		fputc((int)(x), fd); \
	)

#define PORTABLE_PUT_2(fd, x)	\
	STMT( \
		PORTABLE_UPUT_2(fd, (u_int16)(x)); \
	)
	
#define PORTABLE_PUT_4(fd, x)	\
	STMT( \
		PORTABLE_UPUT_4(fd, (u_int32)(x)); \
	)

#define PORTABLE_UPUT_2(fd, x) STMT( \
	fputc((int)((x) >> 8), fd); \
	fputc((int)(x), fd); \
)

#define PORTABLE_UPUT_4(fd, x) STMT( \
	fputc((int)((x) >> 24), fd); \
	fputc((int)((x) >> 16), fd); \
	fputc((int)((x) >> 8), fd); \
	fputc((int)(x), fd); \
)

#define PORTABLE_UGET_2(fd, x) STMT( \
	*(x) = (u_char)getc(fd) << 8; \
	*(x) |= (u_char)getc(fd); \
)

#define PORTABLE_UGET_4(fd, x) STMT( \
	*(x) = (u_char)getc(fd) << 8; \
	*(x) = ((*(x) | (u_char)getc(fd)) << 8); \
	*(x) = ((*(x) | (u_char)getc(fd)) << 8); \
	*(x) |= (u_char)getc(fd); \
)

#define PORTABLE_GET_2(fd, x) STMT( \
	*(x) = (char)getc(fd) << 8; \
	*(x) |= (u_char)getc(fd); \
)

#define PORTABLE_GET_4(fd, x) STMT( \
	*(x) = (char)getc(fd) << 8; \
	*(x) = ((*(x) | (u_char)getc(fd)) << 8); \
	*(x) = ((*(x) | (u_char)getc(fd)) << 8); \
	*(x) |= (u_char)getc(fd); \
)

#ifdef __STDCPP__
#	define PORTABLE_PUT(fd, x, n)	PORTABLE_PUT_##n(fd, x)
#	define PORTABLE_GET(fd, n, x)		PORTABLE_GET_##n(fd, x)
#	define PORTABLE_UPUT(fd, x, n)	PORTABLE_UPUT_##n(fd, x)
#	define PORTABLE_UGET(fd, n, x)		PORTABLE_UGET_##n(fd, x)
#else /* not __STDCPP__ */
#	define PORTABLE_PUT(fd, x, n)	PORTABLE_PUT_/**/n(fd, x)
#	define PORTABLE_GET(fd, n, x)		PORTABLE_GET_/**/n(fd, x)
#	define PORTABLE_UPUT(fd, x, n)	PORTABLE_UPUT_/**/n(fd, x)
#	define PORTABLE_UGET(fd, n, x)		PORTABLE_UGET_/**/n(fd, x)
#endif /* not __STDCPP__ */

/*
 * PORTABLE_PEEK_?() and portable_poke_?() macros:
 *
 * p chould be (char *) or (u_char *), put those macros
 * work even if p is (int *) or any type of pointer.
 * 
 * The returned value of PORTABLE_PEEK_?() is signed.
 * The returned value of portable_upeek_?() is unsigned.
 *
 * Note: Those macros suppose that long and u_long holds at least 32 bits
 * and that "s_char" is for signed char.
 *
 * Comment: I doubt that the ...PEEK_1() and ...POKE_1()
 * macros will be used very often!
 */

#define PORTABLE_PEEK_U_CHAR(p,offset)    (*((u_char *)(p)+(offset)))
#define PORTABLE_PEEK_S_CHAR(p,offset)    (*((s_char *)(p)+(offset)))

#define PORTABLE_PEEK_1(p)		PORTABLE_PEEK_S_CHAR(p,0)

#define PORTABLE_PEEK_2(p)		((PORTABLE_PEEK_S_CHAR(p,0) << 8) \
								| PORTABLE_PEEK_U_CHAR(p,1))

#define PORTABLE_PEEK_4(p)	(((long)PORTABLE_PEEK_S_CHAR(p,0) << 24) \
							|((long)PORTABLE_PEEK_U_CHAR(p,1) << 16) \
							|	   (PORTABLE_PEEK_U_CHAR(p,2) << 8) \
							|		PORTABLE_PEEK_U_CHAR(p,3))

#define PORTABLE_UPEEK_1(p)		PORTABLE_PEEK_U_CHAR(p,0)

#define PORTABLE_UPEEK_2(p)		(PORTABLE_PEEK_U_CHAR(p,0) << 8) \
								| PORTABLE_PEEK_U_CHAR(p,1))

#define PORTABLE_UPEEK_4(p)	(((u_long)PORTABLE_PEEK_U_CHAR(p,0) << 24) \
							|((u_long)PORTABLE_PEEK_U_CHAR(p,1) << 16) \
							|	     (PORTABLE_PEEK_U_CHAR(p,2) << 8) \
							|		  PORTABLE_PEEK_U_CHAR(p,3))

#define PORTABLE_POKE_1(p, x)		\
	STMT( \
		u_char * _p = (u_char *) (p);	\
		_p[0] = (x); \
	)

#define PORTABLE_POKE_2(p, x)		\
	STMT( \
		u_char * _p = (u_char *) (p);	\
		_p[0] = (x) >> 8; \
		_p[1] = (x); \
	)

#define PORTABLE_POKE_4(p, x)		\
	STMT( \
		u_char * _p = (u_char *) (p);	\
		_p[0] = (x) >> 24; \
		_p[1] = (x) >> 16; \
		_p[2] = (x) >> 8; \
		_p[3] = (x); \
	)


#define PORTABLE_UPOKE_1(p, x)		\
	STMT( \
		assert((x) >> 8 == 0); /* assuming sizeof(int) >= 2 */ \
		PORTABLE_POKE_1(p, x); \
	)

#define PORTABLE_UPOKE_2(p, x)		\
	STMT( \
		assert(sizeof(x) <= 2 || (x) >> 16 == 0); \
		PORTABLE_POKE_2(p, x); \
	)

#define PORTABLE_UPOKE_4(p, x)		\
	STMT( \
		assert(sizeof(x) <= 4 || (x) >> 32 == 0); \
		PORTABLE_POKE_4(p, x); \
	)


/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
#ifdef NO_AIO
	typedef char aio_result_t;	/* define dummy type */
#endif

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/

#ifdef HAVE_STDIO_H
extern off_t portable_fsize(FILE *);
extern off_t portable_size(int);

#ifndef TRUNCATE_EXACT

#	define ftruncate(n, size)	portable_ftruncate(n, size)
#	define truncate(name, size)	portable_truncate(name, size)
extern int portable_ftruncate(int, off_t);
extern int portable_truncate(const char *, off_t);

#endif /* not TRUNCATE_EXACT */

#ifdef NO_AIO

#	define aiowrite(fd, bufp, bufs, offset, whence, resultp) \
		portable_ftruncate(fd, bufp, bufs, offset, whence, resultp)
extern int portable_aiowrite(int, const char *, int, off_t, int, aio_result_t *);

#	define aioread(fd, bufp, bufs, offset, whence, resultp) \
		portable_aioread(fd, bufp, bufs, offset, whence, resultp)
extern int portable_aioread(int, char *, int, off_t, int, aio_result_t *);

#	define aiowait(timeout) \
		portable_aiowait(timeout)
extern aio_result_t * portable_aiowait(struct timeval *);
#if 0
//typedef struct timeval _TIMEVAL_KLUDGE;
//extern aio_result_t * portable_aiowait(_TIMEVAL_KLUDGE *);
#endif /* 0 */

#endif /* NO_AIO */

#endif /* HAVE_STDIO_H */

extern void portable_catch_termination_signals(void (*)());


#ifdef NO_HOSTNAME

#	define gethostname(host_name, namelen) \
		portable_gethostname(host_name, namelen)
extern int portable_gethostname(char *, int);

#endif /* NO_HOSTNAME */

#ifdef NO_GETHOSTID

#	define gethostid() \
		portable_gethostid()
extern long portable_gethostid(void);

#endif /* NO_GETHOSTID */

#ifdef HAVE_TIME_H
#ifndef HAVE_UTIME_NULL

	/*
	 * On the Alliant, I am not sure whether 
	 * struct utimbuf {
	 *       time_t    actime;
	 *       time_t    modtime;
	 * };
	 * ... is defined.
	 * If not, we should probably define it here, since it is
	 * used in "portable.c".
	 */

#	define utime(path, times)	portable_utime(path, times)
	extern int portable_utime(const char	*, const struct utimbuf *);

#endif /* not HAVE_UTIME_NULL */

#ifndef HAVE_USLEEP

#	define usleep(usec)	portable_usleep(usec)
	extern void portable_usleep(long);

#define portable_usleep(x) STMT( \
		{ \
			struct timeval	tv; \
			tv.tv_sec = 0; \
			tv.tv_usec = (x); \
			while (tv.tv_usec >= 1000000) { \
				tv.tv_sec++; \
				tv.tv_usec -= 1000000; \
			} \
			select(0, NULL, NULL, NULL, &tv); \
		} \
	)

#endif /* not HAVE_UTIME_NULL */

#endif /* HAVE_TIME_H */

#ifdef READ_AND_WRITE_CAN_RETURN_LESS

#	define read(fd, buf, size)	portable_read(fd, buf, size)
#	define write(fd, buf, size)	portable_write(fd, buf, size)
	extern ssize_t portable_read(int, char *, size_t);
	extern ssize_t portable_write(int, const char *, size_t);

#endif /* READ_AND_WRITE_CAN_RETURN_LESS */

#define anint(x)	portable_nint(x)
#define nint(x)		(int)anint(x)
extern double portable_nint(double);

#if 0
//#ifdef CONFIG_next
//
//#	define stat(path, buf) \
//		portable_stat(path, buf)
//extern int portable_stat(const char	*, struct stat *);
//
//#endif /* CONFIG_next */
#endif /* 0 */


#endif /* _portable_h */
