
#if !defined(_pockettv_h)
#define _pockettv_h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "stdafx.h"

#include "resource.h"
#include "commctrl.h"

#include "mtvp/mtvp.h"

#include "mtvp_player_win32.hpp"

#include "utils.h"
#include "files.h"



/*--------------------------------------------------------------------------+
| Configuration flags
+--------------------------------------------------------------------------*/
// if you need to use those macros, it is safer to define them in the project file.
#if defined(WIN32_PLATFORM_BE300) && !defined(NDEBUG)
//#define POCKETTV_NO_REGISTRATION	// remove registration code
#endif

#define POCKETTV_USE_FULLSCREEN

#ifdef WIN32_PLATFORM_BE300
#define POCKETTV_USE_CSOBAR
#endif

/*--------------------------------------------------------------------------+
| PocketTV Class
+--------------------------------------------------------------------------*/
#define POCKETTV_VERSION_MAJOR		0	// range 0-3 (coded on 2-bit in the registration code)
#define POCKETTV_VERSION_MINOR		6	// range 0-15 (coded on 4-bit in the registration code)
#define POCKETTV_VERSION_REVISION	0	// range 0-15 (coded on 4-bit in the registration code)

typedef enum {
	UNDIM_TYPE_GUESS = -1,
	UNDIM_TYPE_NONE = 0,
	UNDIM_TYPE_HP_540 = 1,
	UNDIM_TYPE_CASIO_115 = 2
} UNDIM_TYPE;

typedef struct {
	UNDIM_TYPE	type;
	unsigned int		timer_interval;
} UNDIM;

typedef enum {
	HARDWARE_PLATFORM_OTHER = 0,
	HARDWARE_PLATFORM_HP720,			// Platform Name "Comanche"
	HARDWARE_PLATFORM_HP820,			// Platform Name "ThunderBird"
	HARDWARE_PLATFORM_IPAQ31XX,			// iPaq 31XX
	HARDWARE_PLATFORM_IPAQ36XX,			// iPaq 36XX
	HARDWARE_PLATFORM_IPAQ37XX,			// iPaq 37XX
	HARDWARE_PLATFORM_IPAQ38XX,			// iPaq 38XX
	HARDWARE_PLATFORM_HP520,
	HARDWARE_PLATFORM_HP540,
	HARDWARE_PLATFORM_HP560,
} HARDWARE_PLATFORM;

typedef enum {
	SOFTWARE_PLATFORM_OTHER = 0,
	SOFTWARE_PLATFORM_RAPIER,
	SOFTWARE_PLATFORM_MERLIN,
	SOFTWARE_PLATFORM_STINGER,
} SOFTWARE_PLATFORM;

typedef enum {
	DISPLAY_TYPE_OTHER = 0,
	DISPLAY_TYPE_PASSIVE_LCD,
} DISPLAY_TYPE;

typedef struct {
	bool display_16bit_write;	// set if vram must be accessed with 16-bit writes
	bool casio_stretch_bug;			// set when StretchBlt is broken (does not do clipping properly)
	bool no_microdrive;				// set for those devices that cannot take a microdrive
	bool display_12bit;				// true if display depth is only 12-bit (e.g. iPaq 36XX)
//	bool ipaq_audio_swapped_channels;// set for bogus devices that swap audio channels (e.g. iPAQ)
	UNDIM_TYPE	undim_type;			// used for determining how to prevent auto dimming
	HARDWARE_PLATFORM hardware_platform;
	SOFTWARE_PLATFORM software_platform;
	DISPLAY_TYPE display_type;

	OSVERSIONINFO os_version_info;

	void *display_address;	// if not NULL, this address will be passed to the GAPI handler.  Used only for some Casio devices.
//	TCHAR oem_info[64]; 
	TCHAR *oem_info;	// if not NULL, contains the string returned by SystemParametersInfo(SPI_GETOEMINFO,...)
} DEVICE_SPEC;

typedef struct {
	bool	grace_period_expired;
	bool	registered;
	bool	purchase_only;	// will only allow purchase (no free key!)
	bool	open_purchase_dialog;
	bool	purchased;
	DWORD	days;	// total grace period (days) before "must register now" or "must purchase now"
	DWORD	dd, hh;	// grace period left before "must register now" or "must purchase now"
} REGISTER_INFOS;

#ifdef POCKETTV_USE_FULLSCREEN

#define IDM_OPTIONS_FULLSCREEN_ROTATION_FIRST Min(IDM_OPTIONS_FULLSCREEN_ROTATION_90L, IDM_OPTIONS_FULLSCREEN_ROTATION_90R)
#define IDM_OPTIONS_FULLSCREEN_ROTATION_LAST Max(IDM_OPTIONS_FULLSCREEN_ROTATION_90L, IDM_OPTIONS_FULLSCREEN_ROTATION_90R)

extern LRESULT CALLBACK 
fullscreen_window_proc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
#endif

class PocketTV : public MTVP_Player_win32
{
public:


typedef enum {
	LAST_OPEN_FROM_NONE,
	LAST_OPEN_FROM_REGISTRY,
	LAST_OPEN_FROM_PARAMETER,
	LAST_OPEN_FROM_PARAMETER_MESSAGE,
	LAST_OPEN_FROM_OPEN_FILE,
	LAST_OPEN_FROM_OPEN_URL,
	LAST_OPEN_FROM_HIBERNATE,
} LAST_OPEN_MODE;

typedef enum {
	FULLSCREEN_ROTATION_90L,
	FULLSCREEN_ROTATION_90R
} FULLSCREEN_ROTATION;

typedef enum {
	NAVBAR_STYLE_OK,
	NAVBAR_STYLE_X
} NAVBAR_STYLE;

typedef enum {
//	DISPLAY_SETTING_BRIGHTNESS,
//	DISPLAY_SETTING_CONTRAST,
	DISPLAY_SETTING_GAMMA,
	DISPLAY_SETTING_FLOOR,
	DISPLAY_SETTING_CHROMA_SATURATION,
	DISPLAY_SETTING_GAMMA_RED,
	DISPLAY_SETTING_GAMMA_GREEN,
	DISPLAY_SETTING_GAMMA_BLUE,
//	DISPLAY_SETTING_GREEN_RED,
//	DISPLAY_SETTING_YELLOW_BLUE,
	DISPLAY_SETTING_COUNT,
} DISPLAY_SETTING;

typedef struct {
	const TCHAR *caption;	// caption
	const TCHAR *registry;	// name of registry value
	int		min;
	int		max;
	int		normal_value;	// range [min..max] - used when reset
	int		default_value;	// range [min..max] - used when reset
	int		value;			// range [min..max]
} DISPLAY_SETTING_INFO;

typedef struct {
	bool	show_status_bar;
	bool	show_stream_url_in_status_bar;
	bool	show_stream_name_rect;	// never true with POCKET_PC
} LAYOUT;


private:
#ifdef POCKETTV_USE_CSOBAR
	HWND				m_CSOBar;
#else
	HWND				m_MenuCommandBar;		// The menu command bar handle
#ifndef POCKET_PC
	HWND				m_CommandBands;		// The command band handle
#endif
//	HWND				m_ControlsCommandBar;	// The command bar handle that hold the slider
#endif
	HWND				m_StatusWindow;		// The status window handle
	LAYOUT				m_Layout;			// Display layout infos
	HMENU				m_Menu;				// The menu bar handle
	TCHAR			   *m_NavBarTitle;		// name of opened stream as shown in navigation bar (when playing)
	TCHAR			   *m_WindowTitle;		// name of opened stream as shown in navigation bar (when playing)
	bool				m_DisplayReady;		// turns True once first image is displayed after a seek
	int					m_PendingSeek;		// -1 or pending seek position
	HWND				m_SeekSlider;		// The track slider handle
	HWND				m_VolumeSlider;		// The volume slider handle
	HWND				m_DisplaySlider;	// The display slider handle
	int					m_DisplaySetting;	// The current display setting being adjusted
	bool				m_PendingDisplaySlider;	// don't set new setting while this is true
	int					m_PendingDisplaySliderPos;	// pending slider position for display setting
	DISPLAY_SETTING_INFO m_DisplaySettingInfo[DISPLAY_SETTING_COUNT];
	MTVP_STATE			m_StateBeforeSeeking;
	MTVP_STATE			m_State;			// current state on player
	int					m_WasPlayingBeforeBecommingInactive; // -1 means unknown (i.e. not yet set)
//	MTVP_STATE			m_StateBeforeCancelMode; // -1 means unknown (i.e. not yet set)
	bool				m_CancelModePending;
	bool				m_SettingVolume;	// used by volume slider procedure
	bool				m_FastForward;		// True if playing fast forward
    WNDPROC				m_VolumeSliderParentOriginalProc;
    WNDPROC				m_DisplaySliderParentOriginalProc;
	WNDPROC				m_SeekSliderParentOriginalProc;
	RECT				m_TargetRect;			// Rectangle for video
	double				m_FilesizeOverTrackSliderMax;
	double				m_TrackSliderMaxOverFilesize; // ratio of TrackSliderMax divided by Filesize
	bool				m_Seeking;			// True means we are currently seeking
	LONGLONG			m_LastPositionTimerTime;	// used by the position timer
	LONGLONG			m_LastVolumeSettingTime;
	bool				m_PlayerReady;		// turns True once the player is fully initialized
	bool				m_Muted;			// audio is muted
	int					m_VOHHandlerId;
	int					m_SIHHandlerId;
	bool				m_IgnoreCancelMode;
//	bool				m_Done;				// set when user clicks the OK button in the Nav Bar
	bool				m_AutoPlay;
	bool				m_Loop;				// if true, should loop when playing file (except if file opened from parameter)
	bool				m_LoopChecked;			// state (checked/unchecked) of the Loop control
	bool				m_FastVideo;		// Will use fast video if possible
	bool				m_FastVideoDisabled;	// do not use Fast video 
	bool				m_GapiDisabled;			// Gapi is not available
	bool				m_FullscreenDisabled;	// Fullscreen not available
	VOH_WIN32_STRETCH	m_Stretch;
	VOH_WIN32_STRETCH	m_FullscreenStretch;
//	UNDIM_TYPE			m_UndimType;
//	unsigned int		m_VolumeRange;		// divisor factor for the volume range (volume is too loud on the iPaq)
	double				m_Volume;			// current Volume (<0 means unknown)
	bool				m_VolumeDialogVisible;
	bool				m_DisplayDialogVisible;
	int					m_DisplayDialogTop;
	HANDLE				m_DisplayThread;			
	bool				m_StreamOpened;		// if false, no stream is currently opened
	REGISTER_INFOS			m_Register;
	bool				m_InOpenDialog;
	TCHAR			   *m_StreamURL;		// URL of opened stream
	TCHAR			   *m_HibernateStreamURL;		// URL of opened stream before hibernation
	RECT				m_StreamNameRect;	// Rectangle to display stream URL above status bar, if necessary
	MTVP_VOH_ROTATION	m_DefaultFullscreenRotation;
	MTVP_VOH_ROTATION	m_FullscreenRotation;
	MTVP_VOH_ROTATION	m_NormalRotation;
//	bool				m_Landscape;			// normal view is in landscape mode
#ifdef USE_VOH_GAPI
	MTVP_VOH_GAPI_INIT_OUTPUT m_GapiInitOutput;
#endif
	bool				m_DisableResumeVideo;
#ifdef POCKETTV_USE_FULLSCREEN
	bool				m_PlayFullscreen;
#endif
	double				m_RateFactor;				// playback rate factor for >>

	MTVP_PMP_DATA_PACK_INFOS m_PackInfos;
	MTVP_PMP_DATA_VIDEO_INFOS m_VideoInfos;
	MTVP_PMP_DATA_AUDIO_INFOS m_AudioInfos;
	unsigned int		m_Filesize;
	unsigned int		m_Position;
	double				m_ActualFramerate;
	DEVICE_SPEC			m_DeviceSpec;
//	bool				m_StartedWithParameter;		// current stream was opened via an .m1u (mpeg url) file for streaming
	TCHAR				*m_TrackbarClass;
//	HWND				m_FirstWindowThatLostFocus;
	HACCEL				m_AccelTable;
	bool				m_IsActive;			// true means main window is active
	bool				m_Microdrive;		// true means optimize for microdrive
	int					m_PendingOpens;		// number of pending "open stream" commands
	unsigned int		m_OpenCount;		// incremented for each stream opened.
	bool				m_UseTGetFile;		// true means use tGetFile if the tgetfile dll is available
	bool				m_UseTGetFileToSave;		// true means use tGetFile if the tgetfile dll is available
	bool				m_RememberLastFile;		// true means try to reopen the last opened file at startup
	HWND 				m_WasForegroundWindow;
//	u_char				m_GapiSuspendFlag;	// if non-zero, will cause the Gapi VOH to stop modifying the display


public:
	HWND 				m_Window;
	HINSTANCE			m_Instance;			// The current instance
	HWND				m_VolumeDialog;		// The volume dialog handle
	HWND				m_DisplayDialog;	// The display dialog handle
//	const TCHAR		   *m_URLName;			// name of last opened URL
	DWORD				m_Code;				// registration code
#ifdef POCKETTV_USE_FULLSCREEN
	bool				m_Fullscreen;
	HWND				m_FullscreenWindow;
#endif
	bool				m_Dither;
	int					m_DitherType;	// MTVP_DITHER_SPEC_12BIT or MTVP_DITHER_SPEC_15BIT
	WNDPROC				m_OriginalTrackbarWindowProc;
	LAST_OPEN_MODE		m_LastOpenMode;


public:
    PocketTV(const char **mtvp_argv, HINSTANCE instance, HWND hWnd) :
		MTVP_Player_win32(mtvp_argv, instance),
		m_Instance(instance),
		m_Window(hWnd),
		m_PendingSeek(-1),
		m_WasPlayingBeforeBecommingInactive(-1),
//		m_StateBeforeCancelMode(MTVP_STATE_UNKNOWN),
		m_CancelModePending(false),

		m_StreamURL(NULL),
		m_HibernateStreamURL(NULL),
#ifdef POCKETTV_USE_CSOBAR
		m_CSOBar(0),
#else
		m_MenuCommandBar(0),
#ifndef POCKET_PC
		m_CommandBands(0),
#endif
//		m_ControlsCommandBar(0),
#endif
		m_StatusWindow(0),
		m_Menu(0),
//		m_StreamNameRect(0),
		m_NavBarTitle(NULL),
		m_WindowTitle(NULL),
		m_DisplayReady(false),
		m_SeekSlider(0),
		m_VolumeSlider(0),
		m_DisplaySlider(0),
		m_DisplaySetting(0),
		m_PendingDisplaySlider(false),
		m_PendingDisplaySliderPos(-1),
		m_StateBeforeSeeking(MTVP_STATE_PAUSED),
		m_State(MTVP_STATE_PAUSED),
		m_SettingVolume(false),
		m_FastForward(false),
		m_VolumeSliderParentOriginalProc(NULL),
		m_DisplaySliderParentOriginalProc(NULL),
		m_SeekSliderParentOriginalProc(NULL),
//		m_TargetRect(0),
		m_FilesizeOverTrackSliderMax(0.0),
		m_TrackSliderMaxOverFilesize(0.0),
		m_Seeking(false),
		m_LastPositionTimerTime(0),
		m_LastVolumeSettingTime(0),
		m_PlayerReady(false),
		m_Muted(false),
		m_VOHHandlerId(MTVP_VOH_HANDLER_ID_WIN32),
		m_SIHHandlerId(-1),
		m_IgnoreCancelMode(false),
//		m_Done(false),
		m_FastVideo(false),
		m_FastVideoDisabled(false),
		m_GapiDisabled(false),
		m_FullscreenDisabled(false),
		m_AutoPlay(false),
		m_Loop(false),
		m_LoopChecked(false),
		m_Stretch(VOH_WIN32_STRETCH_AUTO),
		m_FullscreenStretch(VOH_WIN32_STRETCH_NEVER),
//		m_UndimType(UNDIM_TYPE_GUESS),
//		m_VolumeRange(1),
		m_Volume(-1.0),
		m_VolumeDialogVisible(false),
		m_DisplayDialogVisible(false),
		m_DisplayDialogTop(0),
		m_DisplayThread(0),
		m_StreamOpened(false),
		m_InOpenDialog(false),
		m_DisableResumeVideo(false),
#ifdef POCKETTV_USE_FULLSCREEN
		m_FullscreenWindow(NULL),
		m_PlayFullscreen(false),
		m_Fullscreen(false),
#endif
		m_DitherType(MTVP_COLOR16_DITHER_NONE),
		m_Dither(false),
		m_RateFactor(8),


//		m_PackInfos(0),
//		m_VideoInfos(0),
		m_Filesize(0),
		m_Position(0),
		m_ActualFramerate(0.0),
//		m_PreviouslyActiveWindow(NULL),
		m_WasForegroundWindow(NULL),
		m_VolumeDialog(0),
		m_DisplayDialog(0),
//		m_URLName(NULL),
		m_Code(0),
		m_DefaultFullscreenRotation(MTVP_VOH_ROTATION_90L),
		m_FullscreenRotation(MTVP_VOH_ROTATION_NO),
		m_NormalRotation(MTVP_VOH_ROTATION_NO),
//		m_Landscape(false),
		m_OriginalTrackbarWindowProc(NULL),
		m_TrackbarClass((TCHAR *)TRACKBAR_CLASS),
//		m_StartedWithParameter(false),
		m_LastOpenMode(LAST_OPEN_FROM_OPEN_FILE),
		m_AccelTable(NULL),
		m_IsActive(true),
		m_PendingOpens(0),
		m_OpenCount(0),
		m_Microdrive(false),
		m_UseTGetFile(false),
		m_UseTGetFileToSave(true),
//		m_GapiSuspendFlag(0),
		m_RememberLastFile(false)
//		m_Purchase(false)
//		m_FirstWindowThatLostFocus(NULL)
	{
		memset((void *)&m_StreamNameRect, 0, sizeof(m_StreamNameRect));
		memset((void *)&m_TargetRect, 0, sizeof(m_TargetRect));
		memset((void *)&m_PackInfos, 0, sizeof(m_PackInfos));
		memset((void *)&m_VideoInfos, 0, sizeof(m_VideoInfos));
		memset((void *)&m_AudioInfos, 0, sizeof(m_AudioInfos));
		memset((void *)&m_DeviceSpec, 0, sizeof(m_DeviceSpec));
#ifdef USE_VOH_GAPI
		memset((void *)&m_GapiInitOutput, 0, sizeof(m_GapiInitOutput));
#endif
		memset((void *)&m_Layout, 0, sizeof(m_Layout));
		memset((void *)&m_Register, 0, sizeof(m_Register));
	};
   ~PocketTV();

   bool HandleKeyMessage(UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT SeekSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT VolumeSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT DisplaySliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   BOOL VolumeDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   BOOL DisplayDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	void StartPlayTimers();
	void StopPlayTimers();
	int InitControls();
	void DestroyControls();
	void SetStreamControlsState(bool enabled);
	void ActivateStreamControls();
	void DeactivateStreamControls();
	void OpenURL(const TCHAR *url);
	void OpenURLEx(const TCHAR *url, HWND window, LAST_OPEN_MODE lastOpenMode);
	void OpenFile();
	void CloseStream();
	void Hibernate();
	bool IsStretched(VOH_WIN32_STRETCH stretch, const RECT *pRect);
	void SetVideoStretch0(VOH_WIN32_STRETCH stretch);
	void SetVideoStretchNoWarning(VOH_WIN32_STRETCH stretch);
	void PickVideoMode();
	void SetVideoStretch(VOH_WIN32_STRETCH stretch);
	void SetFullscreenStretchNoWarning(VOH_WIN32_STRETCH stretch);
	void SetFullscreenStretch(VOH_WIN32_STRETCH stretch);
	void SetAudioChannel(MTVP_AUDIO_CHANNEL channel);
	void SetAudioSwapChannelsNoWarning(bool yes);
	void SetAudioSwapChannels(bool yes);
	void SetAudioQuality(MTVP_AUDIO_QUALITY quality);
	void Loop0(bool yes);
	void Loop(bool yes);
	void AutoPlay(bool yes);
	void ShowStatusBar(bool yes);
	int GapiInit();
	int FullscreenInit();
	void GapiSetVideoTargetRect();
	void FullscreenSetVideoTargetRect();
	bool Gapi0(bool yes, bool sync);
	void FastVideo(bool yes);
//	int GapiSuspend();
//	int GapiResume();
	void SetState(MTVP_STATE state);
	void ReflectState(MTVP_STATE state);
	bool WasPlaying();
	int GetElapsedSeconds(unsigned int pos);
	void GetPositionParameters();
	void ShowTimeTotal();
	void ShowTimeElapsed();
	void ShowTimePosition(unsigned int pos);
	void ShowActualFramerate();
	void ShowBitrate();
	void MySeek(unsigned int pos);
	void KeepAlive();
	void CalculateVideoTargetRect();
	void SetVideoTargetRect();
	void UpdateVideoTargetRect();
	void UpdateSlidersSize();
	void UpdateSliderSize(HWND slider, HWND command_bar);
	HWND CreateSlider(HWND window, DWORD style, int x, int y, int width, int height, WNDPROC *window_original_proc, WNDPROC slider_proc);
	HWND CreateSliderInCommandBar(HWND command_bar, unsigned int last_button_index, WNDPROC *command_bar_original_proc, WNDPROC slider_proc);
	void ShowVolumeDialog();
	void HideVolumeDialog();
	void ShowDisplayDialog();
	void HideDisplayDialog();
	void MySetVolume(int volume);
	int ErrorBoxEx(HWND window, int ids_window_caption_error, const TCHAR *format, const va_list *args, u_int type);
	void ErrorBox(HWND window, int ids_window_caption_error, const TCHAR *format, ...);
	int OkCancelBox(HWND window, int ids_window_caption_error, u_int type, const TCHAR *format, ...);
	void SetMute(bool yes);
#ifdef POCKETTV_KEEPBRIGHT
	void KeepBright();
#endif
#ifdef POCKET_PC
	HWND CreateMenuBar(HWND hWnd, int nToolBarId, int dwFlags);
#endif
#ifndef POCKETTV_NO_REGISTRATION
	void InitRegistration(DWORD disposition);
#endif
	void SetUndimType(UNDIM_TYPE type);
//	void SetVolumeRange(unsigned int range);
	void DialogAboutEasterEgg(HWND hWnd);
#ifdef POCKETTV_USE_FULLSCREEN
	void Fullscreen(bool yes);
	void FullscreenKillFocus();
	void FullscreenSetFocus();
	void PlayFullscreen(bool yes);
	void GetFullscreenRect(RECT *pRect1, RECT *pRect2, RECT *pRect3);
	void DefaultFullscreenRotation(MTVP_VOH_ROTATION rotation);
#endif
//	int SuspendVideo0(unsigned int n);
//	int ResumeVideo0(unsigned int n);
	void SuspendVideo();
	void ResumeVideo();
	void Dither(bool yes);
	void SetDither(bool yes);
	void DitherType(int dither);
	void EnableDither(VOH_WIN32_STRETCH stretch);
	void Microdrive(bool yes);
	void SetMicrodrive(bool yes);
	void AllocateMicrodriveBuffer(bool yes);
	void EnableMicrodriveOptimization(bool yes);
	void RateFactor(int rate_factor);
	void SetRateFactor(int rate_factor);
	void FixSliders();
	int GetURLFromM1UFile(const TCHAR *filename, char **purl);
	void HandleMessage(MSG *msg);
//	void HandlePendingMessages();
	void MakeRichInkControl(HWND hDlg, DWORD id, TCHAR *new_text);
	void InitDisplaySettingInfo();
	int CreateDisplayDialog();
	void ShowDisplaySetting();
	void SetDisplaySetting(int pos);
	void SetDisplaySetting0(int i, int pos);
	void SetDefaultDisplaySettings();
	void SetNormalDisplaySettings();
	int Rotate(MTVP_VOH_ROTATION rotation);
	void GetRotation0(MTVP_VOH_ROTATION *fullscreen_rotation, MTVP_VOH_ROTATION *normal_rotation);
	void GetRotation(MTVP_VOH_ROTATION *fullscreen_rotation, MTVP_VOH_ROTATION *normal_rotation);
	void DrawStreamNameRect(HDC hdc);
	void SetHibernateStreamURL(const TCHAR *url);
	void SetStreamURL(const TCHAR *url);
	void SetStreamURLAndUpdateStatusWindow(const TCHAR *url);
	const REGISTER_INFOS *GetRegisterInfos();
	void About();
	void RegisterUpdateMenu();
	void Register(bool open_purchase_dialog);
	void UseTGetFile(bool yes);
	void SetUseTGetFile(bool yes);
	void UseTGetFileToSave(bool yes);
	void SetUseTGetFileToSave(bool yes);
	void RememberLastFile(bool yes);
	int InitInstance(HWND hWnd, HWND wasForegroundWindow, TCHAR *pCommandLine);
	LRESULT DialogRegister(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	BOOL EndDialogRegister(HWND hDlg, int nParam);
	LRESULT DialogProperties(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	int PropInsertItem(HWND list, u_int index, DWORD prop, const TCHAR *value);
	void SetNavBarStyleNoWarning(NAVBAR_STYLE style);
//	void SetNavBarStyle(NAVBAR_STYLE style);
	LONG InitializeRegistrationCode(DWORD *pCode, REGISTER_INFOS *register_infos, DWORD disposition);
	DWORD GetSystemSignatureOld(DWORD sig);
	DWORD GetSystemSignatureNew(DWORD sig);
	DWORD GetSystemSignature(DWORD sig, bool *system_signature_is_based_on_unique_id);
	int SetButtonState(int id, bool enabled);
	int CheckButton(int id, bool checked);
	long IsCheckedButton(int id);
	void VideoRepaint();
	void SaveFrame();


	void MtvpOnNotifyState(MTVP_STATE state);
	void MtvpOnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr);
	void MtvpOnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr);
	void MtvpOnNotifyAudioInfos(const MTVP_PMP_DATA_AUDIO_INFOS *ptr);
	void MtvpOnNotifyFileSize(unsigned int size);
	void MtvpOnNotifyActualFramerate(double x);
	void MtvpOnNotifyVolume(double x);
	void MtvpOnNotifyPosition(int pos);
	void MtvpOnNotifyDisplayReady();
	void MtvpOnNotifyCannotShowVideo(int status);
	void MtvpOnNotifyCannotPlayAudio(int status);
	void MtvpOnNotifyReady();
	void MtvpOnNotifyDisplayThreadId(HANDLE thread);
	void MtvpOnNotifySuccess(MTVP_COMMAND command, void *value, const void *ptr, u_int size);
	void MtvpOnNotifyFailure(MTVP_COMMAND command, int status, const void *ptr, u_int size);
	void MtvpOnNotifyWindowCreated();
};

/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
extern PocketTV *Player;	// that is the only global

/*----------------------------------------------------------------------
|       prototypes
+---------------------------------------------------------------------*/

/*----------------------------------------------------------------------
|       macros
+---------------------------------------------------------------------*/

#define POCKETTV_HANDLE_WINDOW_MESSAGE(object_class,                     \
                                 handler,                          \
                                 window,                           \
						         message,                          \
						         wparam,                           \
						         lparam)                           \
{                                                                  \
	PocketTV *target = Player;                                     \
    if (target) {												   \
        return target->##handler##(window,                         \
		                           message,                        \
		 	    				   wparam,                         \
								   lparam);                        \
	} else {                                                       \
		return DefWindowProc(window, message, wparam, lparam);     \
	}                                                              \
}

extern "C" {
extern LRESULT CALLBACK DialogAbout(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
}

#endif // !defined(_pockettv_h)
