#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/sih_file_win32.c,v 1.3 1999/09/08 08:01:39 tristan Exp tristan $";
static const char copyright[] = "Copyright (c) 1998, MpegTV LLC";
#endif

#include "stdafx.h"

#include "mtvp/mtvp.h"
//#include "xfile.h"

//#include "msg.h"
//#include "msg_types.h"
//#include "sih_file.h"
//#include "get_msg.h"
//#include "ceutils.h"

#include "sih_sample.h"

/*--------------------------------------------------------------------------+
| Configuration flags														|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define MTVP_MSG_PTR	(handler_data->sih_resources->msg)

#define FILE_PROTOCOL "test:"
#define FILE_PROTOCOL_SIZE	(sizeof(FILE_PROTOCOL) - 1)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
typedef struct mtvp_sih_handler_data {
	const MTVP_SIH_RESOURCES *sih_resources;

	struct mtvp_sih_stream_data *stream_data;

} HANDLER_DATA;

typedef struct mtvp_sih_stream_data {
	HANDLER_DATA   *handler_data;
	HANDLE			file_handle;
	HANDLE		    event_handle;
} STREAM_DATA;

/*--------------------------------------------------------------------------+
| file_init																	|
+--------------------------------------------------------------------------*/
static int
file_init(
	const MTVP_SIH_RESOURCES *sih_resources,
	const char *q,
	HANDLER_DATA **handler_data_ptr
)
{
	HANDLER_DATA *handler_data;

	/* allocate storage for our opaque HANDLER_DATA structure */
	handler_data = (HANDLER_DATA *)calloc(1, sizeof(HANDLER_DATA));
	
	if (handler_data == NULL) {
		int status = GetLastError();
		return (status ? status : -1);
	}
	
	handler_data->sih_resources = sih_resources;

	*handler_data_ptr = handler_data;
	return 0;
}

/*--------------------------------------------------------------------------+
| file_probe_url															|
+--------------------------------------------------------------------------*/
static int
file_probe_url(
	const MTVP_SIH_RESOURCES *sih_resources,
	const char *url
)
{	
	if (_strnicmp(url, FILE_PROTOCOL, FILE_PROTOCOL_SIZE)) return -1;

	return 0;
}

/*--------------------------------------------------------------------------+
| file_open_url																|
+--------------------------------------------------------------------------*/
static int
file_open_url(
	HANDLER_DATA *handler_data,
	const char *url,
	STREAM_DATA **stream_data_ptr
)
{
	STREAM_DATA *stream_data = NULL;
	const char *filename;
	HANDLE		file_handle;
	BY_HANDLE_FILE_INFORMATION FileInformation;
	USES_CONVERSION;
	
	filename = url + FILE_PROTOCOL_SIZE;
	
	file_handle = CreateFile(A2W(filename), GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL
#ifndef UNDER_CE
		|FILE_FLAG_SEQUENTIAL_SCAN
#endif
		, NULL);
	if (file_handle == INVALID_HANDLE_VALUE) {
		int status = GetLastError();
//		USES_CONVERSION;
//		MTVP_MSG_ERROR(MTVP_MSG_SIH, ("FILE: error opening file: %s", A2W(filename)));
		goto err;
	}

	/* allocate storage for our opaque STREAM_DATA structure */
	stream_data = (STREAM_DATA *)calloc(1, sizeof(STREAM_DATA));
	
	if (stream_data == NULL) {
		goto err;
	}
	
	stream_data->handler_data = handler_data;
	stream_data->file_handle = file_handle;

	stream_data->event_handle = CreateEvent(NULL, TRUE, TRUE, NULL);
	if (stream_data->event_handle == NULL) {
		goto err;
	}

	if (!GetFileInformationByHandle(file_handle, &FileInformation)) {
		goto err;
	}

	handler_data->stream_data = stream_data;

	*stream_data_ptr = stream_data;
	return 0;
err:
	{
		int status = GetLastError();
		
		if (stream_data != NULL) {
			memory_free(stream_data);
		}
		return (status ? status : -1);
	}
}

/*--------------------------------------------------------------------------+
| file_fd																	|
+--------------------------------------------------------------------------*/
static int
file_fd(
	STREAM_DATA *stream_data,
	void **fd_ptr
)
{	
	*fd_ptr = (void *)stream_data->event_handle;
	return 0;
}

/*--------------------------------------------------------------------------+
| file_close																|
+--------------------------------------------------------------------------*/
static void
file_close(STREAM_DATA *stream_data)
{
	HANDLER_DATA *handler_data = (HANDLER_DATA *)stream_data->handler_data;
	
	if (stream_data->file_handle == INVALID_HANDLE_VALUE) {
	    return;
    }

	CloseHandle(stream_data->event_handle);

	if (!CloseHandle(stream_data->file_handle)) {
//		int status = GetLastError();
//		MTVP_MSG_ERROR(MTVP_MSG_SIH, ("FILE: error closing file"));
	}
	
	/* zeroing before freeing helps catching bugs */
	bzero(stream_data, sizeof *stream_data);
	
	memory_free(stream_data);

	if (stream_data == handler_data->stream_data) {
		handler_data->stream_data = NULL;
	}
}


/*--------------------------------------------------------------------------+
| file_free																	|
+--------------------------------------------------------------------------*/
static void
file_free(HANDLER_DATA *handler_data)
{
	/* zeroing before freeing helps catching bugs */
	bzero(handler_data, sizeof *handler_data);
	
	memory_free(handler_data);
}

/*--------------------------------------------------------------------------+
| file_read																	|
+--------------------------------------------------------------------------*/
static int
file_read(
	STREAM_DATA *stream_data,
	char *buf,
	size_t count,
	size_t *count_read
)
{
	HANDLER_DATA *handler_data = (HANDLER_DATA *)stream_data->handler_data;

		DWORD		n;

		if (!ReadFile(stream_data->file_handle, (LPVOID)buf, (DWORD)count, &n, NULL)) {
			int status = GetLastError();
//			MTVP_MSG_ERROR(MTVP_MSG_SIH, ("FILE: error reading %u bytes from file", count));
			
			return (status ? status : -1);
		}

		if (n == 0) {
			/* reset event on end-of-file */
	//		ResetEvent(stream_data->event_handle);
		}
			
		*count_read = n;
		return 0;
}



/*--------------------------------------------------------------------------+
| file_lseek																|
+--------------------------------------------------------------------------*/
static int
file_seek(
	STREAM_DATA *stream_data,
	off_t offset
)
{
	HANDLER_DATA *handler_data = (HANDLER_DATA *)stream_data->handler_data;
	off_t position;

	position = SetFilePointer(stream_data->file_handle, (LONG)offset, NULL, MTVP_SEEK_SET);
	if (position == 0xffffffff) {
		int status = GetLastError();
//		MTVP_MSG_ERROR(MTVP_MSG_SIH, ("FILE: error seeking in file (pos = %u)", offset));
		return (status ? status : -1);
	}
	return 0;
}


/*--------------------------------------------------------------------------+
| file_size																	|
+--------------------------------------------------------------------------*/
static int
file_size(
	STREAM_DATA *stream_data,
	off_t *size_ptr
)
{
	off_t	size;

	size = GetFileSize(stream_data->file_handle, NULL);

	if (size == 0xffffffff) {
		int status = GetLastError();
		return (status ? status : -1);
	}
	
	*size_ptr = size;
	return 0;
}

/*--------------------------------------------------------------------------+
| file_cntl																	|
+--------------------------------------------------------------------------*/
static int
file_cntl(
	HANDLER_DATA *handler_data,
	const MTVP_SIH_CNTL *ptr,
	u_int size
)
{
	MTVP_SIH_FILE_CNTL *data = (MTVP_SIH_FILE_CNTL *)ptr;
	
	// do what you want here!

	return 0;
}

/*--------------------------------------------------------------------------+
| file_buffer																|
+--------------------------------------------------------------------------*/
static int
file_buffer(
	STREAM_DATA *stream_data,
	MTVP_SIB_IPL_SIZES *sizes
)
{
	const HANDLER_DATA *handler_data = stream_data->handler_data;

	sizes->prefetch_trigger_list = 0;
	sizes->prefetch_trigger_free = 0;

	return 0;
}

/*--------------------------------------------------------------------------+
| SIH_register																|
+--------------------------------------------------------------------------*/
SIH_SAMPLE_API int
SIH_register(MTVP_SIH_EXPOSED *exposed, int version)
{
	if (version == MTVP_SIH_VERSION) {

		exposed->SIH_version = MTVP_SIH_VERSION;
		exposed->SIH_init = file_init;
		exposed->SIH_free = file_free;
		exposed->SIH_probe_url = file_probe_url;
		exposed->SIH_open_url = file_open_url;
		exposed->SIH_close = file_close;
		exposed->SIH_read = file_read;
		exposed->SIH_seek = file_seek;
		exposed->SIH_cntl = file_cntl;
		exposed->SIH_fd = file_fd;
		exposed->SIH_size = file_size;
		exposed->SIH_buffer = file_buffer;
	}
	return MTVP_SIH_VERSION;
}


/* eof */
