// SimplePlayer
//

#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"
#include "simpleplayer.hpp"
#include "files.h"


// WM_NCACTIVATE seems to be missing from the POCKET_PC header files
#ifndef WM_NCACTIVATE
#define WM_NCACTIVATE 0x86
#endif

////////////////////////// SHGetMenu Macro's
#ifdef POCKET_PC //Rapier devices

//Menu Bar Height
#define MENU_HEIGHT 26

#define SHGetMenu(hWndMB)  (HMENU)SendMessage((hWndMB), SHCMBM_GETMENU, (WPARAM)0, (LPARAM)0);
#define SHGetSubMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_GETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);
#define SHSetSubMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_SETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);

//Alternate definition
#define SHMenuBar_GetMenu(hWndMB,ID_MENU) (HMENU)SendMessage((hWndMB), SHCMBM_GETSUBMENU, (WPARAM)0, (LPARAM)ID_MENU);


#else //non-Rapier devices

#define MENU_HEIGHT 0

#define SHGetMenu(hWndCB) (HMENU)CommandBar_GetMenu(hWndCB, 0)
#define SHGetSubMenu(hWndCB,ID_MENU) (HMENU)GetSubMenu((HMENU)CommandBar_GetMenu(hWndCB, 0), ID_MENU)
#define SHSetSubMenu(hWndMB,ID_MENU)


#endif



/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
SimplePlayer *Player;


/*----------------------------------------------------------------------
|       SimplePlayerWindowProc
+---------------------------------------------------------------------*/
LRESULT CALLBACK
SimplePlayerWindowProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
	SimplePlayer *player = Player;
    if (player) {
        return player->HandleWindowsMessage(window,
		                           message,
		 	    				   wparam,
								   lparam);
	} else {
		return DefWindowProc(window, message, wparam, lparam);
	}
}

/*----------------------------------------------------------------------
| HandleWindowsMessage
+---------------------------------------------------------------------*/
LRESULT
SimplePlayer::HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_OPEN:
					OpenFile();
					break;

				case IDM_EXIT:
					SendMessage(hWnd, WM_CLOSE, 0, 0);
					break;
			}
			break;
		case WM_CREATE:
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				BeginPaint(hWnd, &ps);
				EndPaint(hWnd, &ps);
			}
			MtvpRepaint();
			break;
		case WM_SETFOCUS:
			MtvpFocus();
			break;
		case WM_DESTROY:
			MtvpQuit();
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/*----------------------------------------------------------------------
| InitControls
+---------------------------------------------------------------------*/
int
SimplePlayer::InitControls()
{
	int status;
	HWND				menuCommandBar;		// The menu command bar handle

	INITCOMMONCONTROLSEX icex;
	icex.dwSize = sizeof(icex);
	icex.dwICC  = ICC_BAR_CLASSES | ICC_COOL_CLASSES;
	InitCommonControlsEx(&icex);

#ifdef POCKET_PC

	SHMENUBARINFO mbi = {0};

	mbi.cbSize = sizeof (SHMENUBARINFO);
	mbi.hInstRes = m_Instance;
	mbi.hwndParent = m_Window;
	mbi.nToolBarId = IDM_MAIN;
	mbi.nBmpId = 0;
	mbi.cBmpImages = 0;
	mbi.dwFlags = SHCMBF_HIDESIPBUTTON;
	if (!SHCreateMenuBar(&mbi)) {
		menuCommandBar = NULL;
	} else {
		menuCommandBar = mbi.hwndMB;
	}

	if (!menuCommandBar) {
		status = GetLastError();
		_msg_printf("Cannot create menu bar (error %d)\n", status);
		return status ? status : -1;
	}

#elif defined(POCKETTV_USE_CSOBAR)

	CSOBAR_BASEINFO baseInfo = {0};
	baseInfo.x = baseInfo.y = baseInfo.width = baseInfo.height = -1;
	baseInfo.line = 1;
	baseInfo.backColor = CSOBAR_DEFAULT_BACKCOLOR;
	baseInfo.titleColor = CSOBAR_DEFAULT_CODECOLOR;
	
	m_CSOBar = CSOBar_Create(m_Instance, m_Window, 0, baseInfo);

	if (!m_CSOBar) {
		status = GetLastError();
		_msg_printf("Cannot create CSO Bar (error %d)", status);
		return status ? status : -1;
	}

	status = CSOBar_AddAdornments(m_CSOBar, m_Instance, 1, CSOBAR_ADORNMENT_CLOSE, 0);
#else

	HWND commandBands = CommandBands_Create(m_Instance, m_Window, 0,
												RBS_VARHEIGHT | RBS_BANDBORDERS, NULL);

	if (!commandBands) {
		status = GetLastError();
		_msg_printf("Cannot create command bands (error %d)", status);
		return status ? status : -1;
	}
	CommandBands_AddAdornments(commandBands, m_Instance, 0, NULL);
	{
		REBARBANDINFO info[1] = {{0}};

		info[0].cbSize = sizeof(REBARBANDINFO);
		info[0].fMask = RBBIM_ID | RBBIM_STYLE;
		info[0].fStyle = RBBS_NOGRIPPER;
		info[0].wID = IDBAND_MENU;

		CommandBands_AddBands(commandBands, m_Instance, sizeofarray(info), info);
	}

	menuCommandBar = CommandBands_GetCommandBar(commandBands, 0);

	CommandBar_InsertMenubar(menuCommandBar, m_Instance, IDM_MAIN, 0);

#endif

	// play video in nominal size (faster)
	{
		VOH_WIN32_CNTL data;
		data.handler_id = MTVP_VOH_HANDLER_ID_WIN32;
		data.type = VOH_WIN32_CNTL_TYPE_STRETCH;
		data.data.stretch = VOH_WIN32_STRETCH_NEVER;

		MtvpSendCommandCustom(MTVP_COMMAND_VOH_CNTL, (const void *)&data, sizeof data);
	}

	// link our test Stream Input Handler (SIH).
	// to test it:
	// first make sure the dll is built and copied at the correct location on your device.
	// run simpleplayer with a parameter.
	// if the parameter is a URL of the form test:<filename> (e.g. "test:\My documents\file.mpg"), 
	// then it should be opened and read by this custom SIH.
	{
		int		status;
		void	*value;

		// in order to get the return status (and in some cases, the returned value)
		// of an Mtvp command, you must use the synchronous mode as showed here.
		// it is guaranteed that the command has been executed once MtvpGetCommandStatus returns.
		// in general, asynchronous mode is sufficient.
		// when asynchronous mode is used, it is also possible
		// to ask for an synchronous notification of success or failure of the command.
		MtvpSetSynchronousMode();
		MtvpLinkSIH("\\windows\\sih_sample.dll");
		MtvpGetCommandStatus(&status, &value);
		if (status != 0) {
			_msg_printf("MtvpLinkSIH failed, error %d\n", GetLastError());
		}
	}

	return 0;
}

/*----------------------------------------------------------------------
| ParseCommandLine
+---------------------------------------------------------------------*/
static int
ParseCommandLine(TCHAR *cmdline, const TCHAR **argv)
{
	TCHAR *bufp;
	int argc;

	/* Parse a command line buffer into arguments */

	argc = 0;
	for ( bufp = cmdline; *bufp; ) {
		/* Skip leading whitespace */
		while ( iswspace(*bufp) ) {
			++bufp;
		}
		/* Skip over argument */
		if ( *bufp == L'"') {
			++bufp;
			if ( *bufp ) {
				if ( argv ) {
					argv[argc] = bufp;
				}
				++argc;
			}
			/* Skip over word */
			while ( *bufp && (*bufp != L'"') ) {
				++bufp;
			}
		} else {
			if ( *bufp ) {
				if ( argv ) {
					argv[argc] = bufp;
				}
				++argc;
			}
			/* Skip over word */
			while ( *bufp && ! iswspace(*bufp) ) {
				++bufp;
			}
		}
		if ( *bufp ) {
			if ( argv ) {
				*bufp = 0;
			}
			++bufp;
		}
	}
	if ( argv ) {
		argv[argc] = NULL;
	}
	return argc;
}


/*----------------------------------------------------------------------
| GetFirstParam
+---------------------------------------------------------------------*/
const TCHAR *
GetFirstParam(const TCHAR *lpCmdLine)
{
	const TCHAR **argv;
	int argc;
	const TCHAR *bufp;
	TCHAR *cmdline;
	const TCHAR *first_param;

	// return a pointer to a malloc'ed buffer that contains the first
	// parameter of the command line.

	/* Grab the command line */
	bufp = lpCmdLine;

	cmdline = (TCHAR *)malloc(sizeof(TCHAR)*(wcslen(bufp)+1));
	if (!cmdline) return NULL;
	wcscpy(cmdline, bufp);

	/* Parse it into argv and argc */
	argc = ParseCommandLine(cmdline, NULL);

	argv = (const TCHAR **)malloc((argc+1)*(sizeof *argv));
	if (!argv) {
		free(cmdline);
		return NULL;
	}
	ParseCommandLine(cmdline, argv);

	if (argc > 0) {
		first_param = _wcsdup(argv[0]);
	} else {
		first_param = NULL;
	}
	free((void *)cmdline);
	free((void *)argv);

	return first_param;
}

/*----------------------------------------------------------------------
| InitInstance
+---------------------------------------------------------------------*/
static SimplePlayer *
InitInstance(HINSTANCE hInstance, const TCHAR *window_class_name, const TCHAR *window_title, int nCmdShow, TCHAR *pCommandLine)
{
	HWND	hWnd;
	SimplePlayer *player;
	int status;

	{
		int cx, cy;

#ifdef POCKET_PC
		SIPINFO si = {0};

		si.cbSize = sizeof(si);
		SHSipInfo(SPI_GETSIPINFO, 0, &si, 0);

		cx = si.rcVisibleDesktop.right - si.rcVisibleDesktop.left;

		// Consider the menu at the bottom.
		cy = si.rcVisibleDesktop.bottom - si.rcVisibleDesktop.top - MENU_HEIGHT;

		hWnd = CreateWindow(window_class_name, window_title, WS_VISIBLE,
				0, si.rcVisibleDesktop.top, cx, cy, NULL, NULL, hInstance, NULL);

#else
//		cx = CW_USEDEFAULT;
//		cy = CW_USEDEFAULT;

		// the following code is better: it looks at the actual size of the work area
		// which varies depending whether the task bar setting is auto-hide or not.
		// but this code should be used only if the main window resizes properly if
		// the size of the work area changes dynamically... which is not the case here.
		RECT r;

		SystemParametersInfo(SPI_GETWORKAREA, 0, &r, 0);

		cx = r.right - r.left;
		cy = r.bottom - r.top;

		hWnd = CreateWindow(window_class_name, window_title, WS_VISIBLE,
				0, MENU_HEIGHT, cx, cy, NULL, NULL, hInstance, NULL);
#endif


	}
	if (hWnd == 0) {
		return NULL;
	}

	/*
	 * Start the player
	 */

	{
		static const char *mtvp_argv[] = {
			"mtvp",	/* name of the player (ignored on WIN32 platforms), but must be present */
			NULL,	/* must be NULL-terminated */
		};

		// the hWnd specified here is the window used for the message loop
		player = new SimplePlayer(mtvp_argv, hInstance, hWnd);

		if (player == NULL) {
			ExitThread(0);
			return NULL;
		}
	}
	
	// the hWnd specified here is for the display
	player->MtvpSetDisplayWindow((int)hWnd);

	ShowWindow(hWnd, nCmdShow);
	SetForegroundWindow(hWnd);

	status = player->InitControls();
	if (status != 0) {
		ExitThread(0);
		return NULL;
	}

	const TCHAR *file_to_open = GetFirstParam(pCommandLine);
//	const TCHAR *file_to_open = TEXT("test:\\My documents\\levis-bj.mpg");	// you can also use a hard-coded path for testing
	if (file_to_open) {
		USES_CONVERSION;

		player->MtvpOpenStreamURL(W2A(file_to_open));

	} else {

		player->OpenFile();
	}

	return player;
}


/*----------------------------------------------------------------------
| OpenFile
+---------------------------------------------------------------------*/
void
SimplePlayer::OpenFile()
{
	TCHAR name[MAX_PATH];
	USES_CONVERSION;

	if (ChooseOpenFile(m_Window, name, sizeofarray(name),
			FILE_CHOOSER_TYPE_MPEG_VIDEO, TEXT("Open MPEG File"))) {

		MtvpOpenStreamURL(W2A(name));
	}
}

/*----------------------------------------------------------------------
|       WinMain
+---------------------------------------------------------------------*/
int WINAPI
WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;
	const TCHAR	*window_title;			// The title bar text
	const TCHAR	*window_class_name;		// The window class name

	// Initialize global strings
	window_class_name = TEXT("SimplePlayer");
	window_title = TEXT("SimplePlayer");

	{
		WNDCLASS	wc = {0};

		wc.style			= CS_HREDRAW | CS_VREDRAW | CS_NOCLOSE;
		wc.lpfnWndProc		= (WNDPROC) SimplePlayerWindowProc;
		wc.hInstance		= hInstance;
		wc.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
		wc.lpszClassName	= window_class_name;

		if (RegisterClass(&wc) == FALSE) return 0;
	}

	// Perform application initialization:
	Player = InitInstance (hInstance, window_class_name, window_title, nCmdShow, lpCmdLine);

	if (Player == NULL) {
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_SIMPLEPLAYER);


	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {

		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	return msg.wParam;
}

// eof
